BOOTSTRAP_CSS = """
QPushButton {
    background-color: #0d6efd;
    color: white;
    border: none;
    padding: 6px 12px;
    border-radius: 4px;
    font-size: 14px;
    font-weight: bold;
}

QPushButton:hover {
    background-color: #0b5ed7;
}

QPushButton:pressed {
    background-color: #0a58ca;
}

QTableWidget {
    font-size: 12px;
    font-weight: bold;
    color: slateblue;
}

QHeaderView::section {
    color: darkslateblue;
    font-size: 14px;
    font-weight: bold;
    padding: 4px;
}

"""